
DB的实现方案

一､简介

此持久层框架主要用于支撑Sqlite与Spatialite两种数据库的CRUD操作。

由于现有的框架，如ORMLite, GreenDAO等，均不支持Spatialite数据库，故无法采用现有的框架直接进行数据迁移。且对于
Spatialite采用的是JSQLITE的实现方式，直接通过操作DB执行数据操作，对于批量数据，并未提供相应的批量操作接口，导致数
据的修改效率较低。而通过查看源码发现，JSQLITE 提供了JDBC驱动（JDBC原生支持批量数据操作，且JDBC对多线程操作有较好
的支持。故通过采用JDBC的方式统一对于DB的访问（对于SQLite采用SQLDroid的实现并同样JDBC的方式）。


二､结构

1、DBHelper

DBHelper类为对外接口类，提供各相关的IMasterOpera, 用于操作各DB
同时在DBHelper中注册各种实体类，实现Table的自动创建。注册时，需要指明TABLEKE，当作使用IMasterOpera执行CRUD操
作的入参，同时作为MappingDao的Key.

2、DaoMaster
各个DaoMaster为DB管理类，对于SQLite，DaoMaster实现对DB的创建，更新等操作；对于SPATIALite(不需要进行DB的创建)，
实现了对于加密的操作。

具体的实现主要包含在AbstractSQLiteMaster和AbstractSpetialiteMaster中。
AbstractSQLiteMaster有一个实现类CommonDaoMaster。
AbstractSpetialiteMaster 有两个实现类RoadDaoMaster与TrackDaoMaster。

3、JDBCDao具体的CRUD执行者
JDBCDao中封装了JDBC执行CRUD的具体逻辑。

4、Column, ColumnRule, Table
Column 用于描述字段
ColumnRule 用于描述字段的特殊规则
Table  用于描述表

5、ColumnEntity, DaoConfig, TableEntity
ColumnEntity 字段相关信息
DaoConfig 表， SQLBUILDE， 表的对应关系
TableEntity  表的信息

三、实现

1、IMasterOper
此接口为DaoMaster中暴露给使用者使用的接口，实现了对DAO所实现的ICRUDOPERA接口的封装。从而避免将DAO直接暴露给使用者。

    long insert(TableKey table, T bean);

    void insert(TableKey table, List<T> bean);

    int delete(TableKey table, String whereClause, String[] whereArgs);

    int update(TableKey table, Map<String, Object> values, String whereClause, String[] whereArgs);//单条更新

    boolean update(TableKey table, List<Map<String, Object>> values,List<String> whereArgs);//批量更新

    List<T> list(TableKey table, String whereClause, String[] whereArgs);

    ResultSet query(TableKey table, String sql);

    void exec(TableKey table, String sql);


2、DaoMaster的实现
DaoMaster分为两种，分别实现了对SQLITE与SPATIALITE两种DB的支持。
其中，AbstractSQLiteMaster实现了对SQLITE的支持，AbstractSpetialiteMaster实现了对SPATIALITE的支持。


（1）、AbstractSQLiteMaster类
AbstractSQLiteMaster类继承了SQLiteOpenHelper， 用来实现对DB 创建与升级的处理， 及返回IMASTEROPERA接口。

重要方法：
register(Class<T> cls, TableKey table) 此方法将BEAN与BEAN的描述信息进行绑定， 生成BEAN所对应的DAOCONFIG对
象，并进行存储。

public void registerDaos(); 注册过的BEAN所对应的DAO对象，并对对象进行存储。此方法必须在register方法之后执行，
在此方法中会调用getPath()方法，而执行getWritableDatabase()方法，而激活DB的创建或升级操作。

IMasterOpera getOperation()  返回IMasterOper，以供使用者调用，在执行相关CRUD操作时，会需要传入相应的TABLEKE，
此TABLEKE即为调用register方法时所指定。

    static {
        try {
            Class.forName("org.sqldroid.SQLDroidDriver").newInstance();
        }catch (IllegalAccessException e) {
            e.printStackTrace();
        } catch (InstantiationException e) {
            e.printStackTrace();
        } catch (ClassNotFoundException e) {
            e.printStackTrace();
        }
    }

    protected static final String JDBC_DRIVER_SUFFIX = "jdbc:sqlite:";

    //调用必须在所有register之后，因可能执行DB创建操作。
    public void registerDaos()

    public  <T extends AbstractEntity> void register(Class<T> cls, TableKey table);
    protected String getPath();
    public IMasterOpera getOperation();



（2）、AbstractSpetialiteMaster

此类实现对SPATIALITE数据库操作的封装。由于SPATIAL数据库由下载而来，故不需要本地创建，因此创建操作处于未调用状态。

重要方法：
register(Class<T> cls, TableKey table)  同上；

IMasterOpera getOperation()   同上；

对于AbstractSpetialiteMaster的实现类，还需要重写KE（）方法，用于对SPATIALITE数据库加密操作的处理。

    static {
        try {
            Class.forName("jsqlite.JDBCDriver").newInstance();
        }catch (IllegalAccessException e) {
            e.printStackTrace();
        } catch (InstantiationException e) {
            e.printStackTrace();
        } catch (ClassNotFoundException e) {
            e.printStackTrace();
        }
    }

    private static final String JDBC_DRIVER_SUFFIX = "jdbc:jsqlite:/";

    public final Map<Class<? extends AbstractEntity>, DaoConfig> entityToDao = new HashMap<>();
    public final Map<TableKey, JDBCDao> daoMapping = new HashMap<>();

    public  <T extends AbstractEntity> void register(Class<T> cls, TableKey key) ;
    public String getPath();
    public Properties getProperities();
    public void close();
    public IMasterOpera getOperation();


区别于AbstractSQLiteMaster， 无需要调用registerDaos。

public AbstractSpetialiteMaster(String path, boolean canCreate) 其中，onCreate参数表示是否需要创建Table， 通过registerDaos来创建Table.


在Master中有对两种DB ， JDBC DRIVER注册的过程


3、JDBCDao

（1）、ICrudOpera

    boolean insert(List<T> bean);

    int delete(String whereClause, String[] whereArgs);

    int update(Map<String, Object> values, String whereClause, String[] whereArgs);//单条更新

    boolean update(Map<String, Object> values, String whereClause, List<String[]> whereArgs);//批量更新

    List<T> list(String whereClause, String[] whereArgs);

    ResultSet query(String sql);

    boolean exec(String sql);


（2）、JDBCDao
此类为具体的CRUD操作实现类， 实现了ICrudOpera接口
构造函数据参数由DB DRIVER(由DAO中的getPath方法获得)和BEAN对应的DAOCONFIG构成。DRIVER主要供JDBC获取DB连接。
DAOCONFIG 主要用来获取BEAN 的信息以及获取SQLBUILDER产生SQL语句。

问题1：
DriverManager.getConnection() 与DriverManager.getDriver(path).connect(path, properties)
通过第一条语句所获取的Connection总是因具体的实现，而返回SQLITE对应的Connectio， 而非SPATIALITE的，故采用后者。


     Connection conn = DriverManager.getDriver(path).connect(path, properties);


问题2：
若采用同一个Connection操作一个DB ，是否会更合理？
若采用同一个Connection操作一个TABLE， 会因有多个表同时操作，会因事务问题而产生问题，故在执行操作时，会通过
DriverManager来重新获取Connection，使用完毕后再关闭Connection，回收。

     if(null != conn)
        conn.close();

问题3：
在插入GEOM数据时，因使用的setString的方式进行填充，Driver会自动添加引号，导致插入数据失败！
解决办法：将对GEOMFORMSTRIN写于SQL语句中，仅把数据作为部分填充。


4、注解的使用
（1）、Column

此注解用于标识字段，包含一些描述字段特殊的属性。、

    String name();
    boolean isAutoGen() default false;
    boolean isKey() default false;
    boolean canBeNull() default true;
    String type() default "";
    boolean isUnique() default false;
    ColumnRule rule() default @ColumnRule;

（2）、Table
用于描术表的信息，如表名，是否创建自动增长的Key两胀属性

    String name();
    boolean autoID() default false;  //添加标记用于判断是否自动添加ID

3、ColumnRule
用于处理，字段的特殊逻辑，如加密（DE），查询时的显示名（GEO）等

    String valueOpea() default "";//对于数据的保作，用于对加密解密类操作等
    String list() default "";//执行LIST操作时，用于显示的名字，如GEOASTEXT


四、操作示例

    DBHelper helper = DBHelper.getInstance(mContext);
    IMasterOpera roadOpear = DBHelper.getInstance(context).getRoadDaoOpera();
    1、插入操作
    roadOpear.insert(TableKey.ROAD_KEY, list);
    roadOpear.insert(TableKey.ROAD_KEY, bean);
    对于单条插入操作，若ID（KE）为自动生长的类型，则返回ID ，否则，返回状态值（1或-1， 分别代表成功与失败）


    2、查询操作
    roadOpear.list(TableKey.ROAD_KEY, null, null);//select all

    3、更新操作
     Map<String, Object> values = new HashMap<String, Object>();
     values.put("taskId", 1110);
     values.put("status", 1110);
     roadOpear.update(TableKey.ROAD_KEY， values, "taskId=?", new String[]{"111"});

     批量更新
     List<Map<String, Object>> values = new ArrayList<Map<String, Object>>();
     Map<String, Object>  map1 = new HashMap<String, Object>();
     map1.put("errorID", 110);
     Map<String, Object>  map2 = new HashMap<String, Object>();
     map2.put("errorID", 114);
     values.add(map1);
     values.add(map2);

     List<String> whereCalses = new ArrayList<String>();

     whereCalses.add("roadID=110");
     whereCalses.add("roadID=114");

     boolean result = opera.update(TableKey.ERROR_INFO, values, whereCalses);

     4、删除操作

     roadOpear.delete(TableKey.ROAD_KEY， "taskId=?", new String[]{"1110"});
     roadOpear.delete(TableKey.ROAD_KEY， "1=1", null);//delete all

五、数据库创建与升级的逻辑
1、由于Spatialite数据库由服务下发，数据格式基本不变，故暂无需处理。
如果需要特殊处理，需要执行PRAGMA user_version 指令，执行相应的版本获取与设置操作。


2、对于Sqlite类型的DB ，数据的处理逻辑与标准API相同。
数据创的创建、升级与降级逻辑由具体的DaoMaster来控制，分别是

    public void onCreate(android.database.sqlite.SQLiteDatabase db)；
    public void onUpgrade(android.database.sqlite.SQLiteDatabase sqLiteDatabase, int i, int i1)；
    public void onDowngrade(SQLiteDatabase db, int oldVersion, int newVersion)

由于SQLite中存储的数据变更的频率比较高，BEAN的变动也比较大。如BEAN变更的情况，下需要执行升级操作，及时更新DB VERSION。
且由于DB为根据BEAN的属性自动创建，故如果属性名增加或变更，建议为属性名添加Column注解，以便和DB统一。

若BEAN的结构有变更，如增加新的属性等，需要在onUpgrade中执行相关的ALTER操作。
若BEAN结构变更，减少了属性，无需要执行升级逻辑，表结构不需调整。




缓存层及业务层的包装
1、现实方式
    具体的业务逻辑主要位于persistence.wrapper包下，BaseWrapper, CacheWrapper, RoadWrapper, TrackWrapper,
CommonWrapper.提供对数据层与业务逻辑之间的封装。
    其中，BaseWrapper提供对基本的DB逻辑，可获取DBHelper与IMasterOpera DB存储接口。
         CaccheWrapper 继承自BaseWrapper, 提供对缓存的支持。
         CommonWrapper 继承自BaseWrapper, 实现了对CommonMaster的封装，可提供针对CommonMaster对应的IMasterOPera
    接口。同时封装了业务层与CommonMaster之间的数据包装与存储逻辑。
        RoadWrapper， TrackWrapper 继承自CacheWrapper, 实现了对RoadMaster, TrackMaster的封装，可提供针
    对RoadMaster, TrackMaster对应的IMasterOPera接口


2、使用方式
    DataManager为数据管理工厂，可从工厂中获取对RoadWrapper, TrackWrapper, CommonWrapper引用，从而通过相关接口
进行调用，并实现数据的缓存，包装，封装，解包装与存储逻辑。




